// This file is part of the program FRYSK.
//
// Copyright 2008, Red Hat Inc.
//
// FRYSK is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// FRYSK is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with FRYSK; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
// 
// In addition, as a special exception, Red Hat, Inc. gives You the
// additional right to link the code of FRYSK with code not covered
// under the GNU General Public License ("Non-GPL Code") and to
// distribute linked combinations including the two, subject to the
// limitations in this paragraph. Non-GPL Code permitted under this
// exception must only link to the code of FRYSK through those well
// defined interfaces identified in the file named EXCEPTION found in
// the source code files (the "Approved Interfaces"). The files of
// Non-GPL Code may instantiate templates or use macros or inline
// functions from the Approved Interfaces without causing the
// resulting work to be covered by the GNU General Public
// License. Only Red Hat, Inc. may make changes or additions to the
// list of Approved Interfaces. You must obey the GNU General Public
// License in all respects for all of the FRYSK code and other code
// used in conjunction with FRYSK except the Non-GPL Code covered by
// this exception. If you modify this file, you may extend this
// exception to your version of the file, but you are not obligated to
// do so. If you do not wish to provide this exception without
// modification, you must delete this exception statement from your
// version and license this file solely under the GPL without
// exception.

#include <errno.h>
#include <linux/unistd.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <sys/syscall.h>
#include <string.h>

#include "jni.hxx"

#include "jnixx/exceptions.hxx"
#include "jnixx/elements.hxx"

void
frysk::sys::Signal::kill(::jnixx::env env, jint pid, jint sig,
                         ::java::lang::String name) {
  errno = 0;
  if (::kill(pid, sig) < 0) {
    const char *sname = name.GetStringUTFChars(env);
    if (sname == NULL)
      return; // exception pending
    errnoException(env, errno, "kill", "process %d, signal %s (%d)",
    		   pid, sname, sig);
    name.ReleaseStringUTFChars(env, sname);
  }
}

void
frysk::sys::Signal::tkill(::jnixx::env env, jint tid, jint sig,
                          ::java::lang::String name) {
  errno = 0;
  if (::syscall(__NR_tkill, tid, sig) < 0) {
    jstringUTFChars sname = jstringUTFChars(env, name);
    errnoException(env, errno, "tkill", "task %d, signal %s (%d)",
    		   tid, sname.elements(), sig);
    sname.release(); // happens during unwind
  }
}

void
frysk::sys::Signal::drain(::jnixx::env env, jint sig) {
  struct sigaction oldAction = { };
  struct sigaction newAction = { };
  newAction.sa_handler = SIG_IGN;
  if (::sigaction(sig, &newAction, &oldAction) < 0) {
    errnoException(env, errno, "sigaction", "signal %d - %s",
                   sig, strsignal(sig));
    return;
  }
  if (::sigaction(sig, &oldAction, NULL) < 0) {
    errnoException(env, errno, "sigaction", "signal %d - %s",
                   sig, strsignal(sig));
    return;
  }
}

jint
frysk::sys::Signal::nsig(::jnixx::env env) {
  return NSIG;
}

jint
frysk::sys::Signal::rtMin(::jnixx::env env) {
  return SIGRTMIN;
}

jint
frysk::sys::Signal::rtMax(::jnixx::env env) {
  return SIGRTMAX;
}

jint
frysk::sys::Signal::hup(::jnixx::env ) {
#ifdef SIGHUP
    return SIGHUP;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::int_(::jnixx::env ) {
#ifdef SIGINT
    return SIGINT;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::quit(::jnixx::env ) {
#ifdef SIGQUIT
    return SIGQUIT;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::ill(::jnixx::env ) {
#ifdef SIGILL
    return SIGILL;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::abrt(::jnixx::env ) {
#ifdef SIGABRT
    return SIGABRT;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::fpe(::jnixx::env ) {
#ifdef SIGFPE
    return SIGFPE;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::kill(::jnixx::env ) {
#ifdef SIGKILL
    return SIGKILL;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::segv(::jnixx::env ) {
#ifdef SIGSEGV
    return SIGSEGV;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::pipe(::jnixx::env ) {
#ifdef SIGPIPE
    return SIGPIPE;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::alrm(::jnixx::env ) {
#ifdef SIGALRM
    return SIGALRM;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::term(::jnixx::env ) {
#ifdef SIGTERM
    return SIGTERM;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::usr1(::jnixx::env ) {
#ifdef SIGUSR1
    return SIGUSR1;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::usr2(::jnixx::env ) {
#ifdef SIGUSR2
    return SIGUSR2;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::chld(::jnixx::env ) {
#ifdef SIGCHLD
    return SIGCHLD;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::cont(::jnixx::env ) {
#ifdef SIGCONT
    return SIGCONT;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::stop(::jnixx::env ) {
#ifdef SIGSTOP
    return SIGSTOP;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::tstp(::jnixx::env ) {
#ifdef SIGTSTP
    return SIGTSTP;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::ttin(::jnixx::env ) {
#ifdef SIGTTIN
    return SIGTTIN;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::ttou(::jnixx::env ) {
#ifdef SIGTTOU
    return SIGTTOU;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::bus(::jnixx::env ) {
#ifdef SIGBUS
    return SIGBUS;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::poll(::jnixx::env ) {
#ifdef SIGPOLL
    return SIGPOLL;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::prof(::jnixx::env ) {
#ifdef SIGPROF
    return SIGPROF;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::sys(::jnixx::env ) {
#ifdef SIGSYS
    return SIGSYS;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::trap(::jnixx::env ) {
#ifdef SIGTRAP
    return SIGTRAP;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::urg(::jnixx::env ) {
#ifdef SIGURG
    return SIGURG;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::vtalrm(::jnixx::env ) {
#ifdef SIGVTALRM
    return SIGVTALRM;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::xcpu(::jnixx::env ) {
#ifdef SIGXCPU
    return SIGXCPU;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::xfsz(::jnixx::env ) {
#ifdef SIGXFSZ
    return SIGXFSZ;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::iot(::jnixx::env ) {
#ifdef SIGIOT
    return SIGIOT;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::emt(::jnixx::env ) {
#ifdef SIGEMT
    return SIGEMT;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::stkflt(::jnixx::env ) {
#ifdef SIGSTKFLT
    return SIGSTKFLT;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::io(::jnixx::env ) {
#ifdef SIGIO
    return SIGIO;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::cld(::jnixx::env ) {
#ifdef SIGCLD
    return SIGCLD;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::pwr(::jnixx::env ) {
#ifdef SIGPWR
    return SIGPWR;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::info(::jnixx::env ) {
#ifdef SIGINFO
    return SIGINFO;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::lost(::jnixx::env ) {
#ifdef SIGLOST
    return SIGLOST;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::winch(::jnixx::env ) {
#ifdef SIGWINCH
    return SIGWINCH;
#else
    return -1;
#endif
}

jint
frysk::sys::Signal::unused(::jnixx::env ) {
#ifdef SIGUNUSED
    return SIGUNUSED;
#else
    return -1;
#endif
}
